#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_n $opt_v $opt_o);
&getopts('hn:v:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -v   release version of Ensembl genome browser
# -n   an organism name
# -o   a directory in which the downloaded file is reposited

## Others:
# -h   print help

_EOH_
    ;

die $usage if $opt_h;

# Get input parameters from command line
my $orgname = $opt_n or die $usage;
my $version = $opt_v or die $usage;
my $dwn_dir = $opt_o or die $usage;
$dwn_dir   .= '/' unless $dwn_dir =~ /\/$/;
my $org_dir = $dwn_dir . $orgname . '.v' . $version . '/';
system "mkdir $org_dir" unless (-d $org_dir);
my $dwnfile = $org_dir . 'ensembl_mfa.gz';

# Download MFA file from Ensembl genome browser
my $enspath = "ftp://ftp.ensembl.org/pub/"
    . 'release-' . $version . '/'
    . 'fasta/'
    . $orgname . '/'
    . 'pep/'
    . '*.all.fa.gz';
`wget $enspath -O $dwnfile`;
if( $dwnfile =~ /\.gz$/ ){
    system "gunzip $dwnfile";
}
