#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_m $opt_n $opt_o);
&getopts('hm:n:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -m   mode [0:general mode, 1:bacteria mode, 2:fungi mode, 3:protozoa mode]
# -n   an organism name
# -o   a directory in which the downloaded data is reposited 

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
die $usage if !defined($opt_m);
my $mode    = $opt_m;
my $orgname = $opt_n or die $usage;
my $rep_dir = $opt_o or die $usage;
$rep_dir   .= '/' unless $rep_dir =~ /\/$/;
die "$rep_dir is not valid directory." unless( -d $rep_dir );
my $org_dir = $rep_dir . $orgname . '/';
system "mkdir $org_dir" unless (-d $org_dir);

# Get a list of the organism names from the NCBI ftp site
my $tmpfile = 'orglist_ncbi.'.$$;
my $nc_path = "ftp://ftp.ncbi.nih.gov/genomes/";
$nc_path   .= 'Bacteria/' if $mode == 1;
$nc_path   .= 'Fungi/'    if $mode == 2;
$nc_path   .= 'Protozoa/' if $mode == 3;
$nc_path   .= $orgname . '/';
my $cmd = 'ftp ' . $nc_path . '> ' . $tmpfile . ' << _EOF_' . "\n"
    . 'ls' . "\n"
    . 'bye' . "\n"
    . '_EOF_' . "\n";
print STDERR "Connecting to " . $nc_path . "...\n";
system($cmd);

# Get GBK files
get_gbk_files( $nc_path, $tmpfile, $org_dir );

# Get GBK files in the chromosome directories
get_chrom_files( $nc_path, $tmpfile, $org_dir );

system("rm $tmpfile");


sub get_gbk_files {
    my $nc_path = shift;
    my $file    = shift;
    my $out_dir = shift;
    $nc_path   .= '/' unless $nc_path =~ /\/$/;
    $out_dir   .= '/' unless $out_dir =~ /\/$/;
    open(IN, $file) or die "Can't open $file to read.";
    while(<IN>){
	chomp;
	if(/ftp/){
	    my @cols = split(" ", $_);
	    my $file_name = $cols[-1];
	    if( $file_name=~/\.gbk$/ or $file_name=~/\.gbk\.gz$/ ){
		print STDERR "Downloading $file_name...";
		my $path    = $nc_path . $file_name;
		my $dwnfile = $out_dir . $file_name;
		system "wget $path -nc -O $dwnfile > /dev/null >& /dev/null";
		if( $dwnfile =~ /\.gz$/ ){
		    system "gunzip $dwnfile";
		}
		print STDERR "done\n";
	    }
	}
    }
    close IN;
}


sub get_chrom_files {
    my $nc_path = shift;
    my $file    = shift;
    my $out_dir = shift;
    $nc_path   .= '/' unless $nc_path =~ /\/$/;
    $out_dir   .= '/' unless $out_dir =~ /\/$/;
    open(CHR, $file) or die "Can't open $file to read.";
    while(<CHR>){
	chomp;
	if(/^d.+ftp/){
	    my @cols = split(" ", $_);
	    my $dir_name = $cols[-1];
	    if( $dir_name=~/^CHR_(.+)$/i ){
		my $tmpfile = 'ncbi.chrom.'.$1.$$;
		my $chrPath = $nc_path . $dir_name;
		$chrPath   .= '/' unless $chrPath =~ /\/$/;
		my $cmd = 'ftp '.$chrPath.'> '.$tmpfile.' << _EOF_'."\n"
		    . 'ls' . "\n"
		    . 'bye' . "\n"
		    . '_EOF_' . "\n";
		print STDERR "Connecting to " . $chrPath . "...\n";
		system($cmd);

		get_gbk_files( $chrPath, $tmpfile, $out_dir );
		system "rm $tmpfile";
	    }
	}
    }
    close CHR;
}
