#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_i $opt_d $opt_o);
&getopts('hi:d:o:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -i   a directory for organism A (used as query)
# -d   a directory for organism B (used as database)
# -o   a directory for outputting the result of BLASTP

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
my $q_dir   = $opt_i or die $usage;
my $d_dir   = $opt_d or die $usage;
my $out_dir = $opt_o or die $usage;
$q_dir     .= '/' unless $q_dir =~ /\/$/;
$d_dir     .= '/' unless $d_dir =~ /\/$/;
$out_dir   .= '/' unless $out_dir =~ /\/$/;

# Set file names
my $q_seq   = $q_dir . 'all_proteins.mfa';
my $d_seq   = $d_dir . 'all_proteins.mfa';
die "No mfa file is found in $q_dir." unless (-f $q_seq);
die "No mfa file is found in $d_dir." unless (-f $d_seq);

# Format database
unless( (-f $d_seq.'.phr') && (-f $d_seq.'.pin') && (-f $d_seq.'.psd')
	&& (-f $d_seq.'.psi') && (-f $d_seq.'.psq') ){
    my $format_cmd = "formatdb -i $d_seq -p T -o T";
    print STDERR "Formatting database ($d_seq)...\n";
    system $format_cmd;
}

# Execute BLASTP
print STDERR "Executing the BLASTP program..." . "\n";
my @q_str   = split("/", $q_dir);
my $q_name  = $q_str[-1];
my @d_str   = split("/", $d_dir);
my $d_name  = $d_str[-1];
my $outFile = $out_dir . $q_name . '.vs.' . $d_name . '.blastp';
my $bls_cmd = "blastall -p blastp -i $q_seq -d $d_seq -o $outFile -m 9";
system $bls_cmd;
