#!/usr/bin/perl -w

use strict;
use Data::Dumper;
use Getopt::Std;
use Storable qw(dclone);

use vars qw ($opt_h $opt_m);
&getopts('hm:');

my $usage = <<_EOH_;

## Options ###########################################
## Required:
# -m   mode [0:general mode, 1:bacteria mode, 2:fungi mode, 3:protozoa mode]

## Others:
# -h   print help

_EOH_
    ;

# Get input parameters from command line
die $usage if $opt_h;
die $usage if !defined($opt_m);
my $mode    = $opt_m;

# Get a list of the organism names from the NCBI ftp site
my $tmpfile = 'orglist_ncbi.'.$$;
my $nc_path = "ftp://ftp.ncbi.nih.gov/genomes/";
$nc_path   .= 'Bacteria/' if $mode == 1;
$nc_path   .= 'Fungi/'    if $mode == 2;
$nc_path   .= 'Protozoa/' if $mode == 3;
my $cmd = 'ftp ' . $nc_path . '> ' . $tmpfile . ' << _EOF_' . "\n"
    . 'ls' . "\n"
    . 'bye' . "\n"
    . '_EOF_' . "\n";
print STDERR "Connecting to " . $nc_path . "...\n";
system($cmd);

# Print a list of the organism names obtained from the NCBI ftp site
open(IN, $tmpfile) or die "Can't open $tmpfile to read.";
my $orgID = 0;
print '------ ------------------------------------------' . "\n"
    . ' ID     organism name'                             . "\n"
    . '------ ------------------------------------------' . "\n";
while(<IN>){
    chomp;
    if(/^d.+ftp/){
	my @cols = split(" ", $_);
	if( $mode==0 ){
	    my $dir_name = $cols[-1];
	    my @name = split("_", $dir_name);
	    if( @name == 2 ){
		print ++$orgID . "\t" . $cols[-1] . "\n";
	    }
	} elsif( $mode==1 or $mode==2 or $mode==3 ){
	    print ++$orgID . "\t" . $cols[-1] . "\n";	    
	}
    }
}
close IN;
system("rm $tmpfile");
