// *************************************************************************
//  options.cpp
// *************************************************************************

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "osfinder.h"

// *************************************************************************
// Function that phases command.

input option_disposal( int argc, char* argv[] ){
  input in;
  char opt;
  char tmp_str[N_STR];
  int opt_idx;
  bool flag_optI = false;
  bool flag_optO = false;
  bool flag_optN = false;
  bool flag_optS = false;
  bool flag_optH = false;

  std::string usage;
  usage = (std::string)"\n"
    + "********************************************************"  + "\n"
    + "Description:"                                              + "\n"
    + "  OSfinder -- a tool for accurate orthology mapping"       + "\n"
                                                                  + "\n"
    + "Options:"                                                  + "\n"
    + "  -i  set file containing the coordinate span for anchors" + "\n"
    + "  -o  set output name"                                     + "\n"
    + "  -n  set the number of genomes under comparison"          + "\n"
    + "  -s  set the minimum length of orthologous segments"      
    + " (default=100000)"                                         + "\n"
    + "  -L  set \"global maximization method\""
    + " as the parameter optimization method"                     + "\n"
    + "      (default is \"local maximization method\")"          + "\n"
    + "  -S  assume that input anchors are sorted according to"
    + " its start coordinate"                                     + "\n"
    + "  -D  output detailed results"                             + "\n"
    + "  -h  print help message"                                  + "\n"
    + "********************************************************"  + "\n";
    
  in.set_sb_size( DEFAULT_SB_SIZE );
  in.set_flagL( true );
  in.set_flagS( false );
  in.set_flagD( false );

  opt_idx = 0;
  while( -1 ){
    opt = getopt( argc, argv, "i:o:n:s:LSDh" );
    opt_idx++;
    if( opt == -1 ){
      break;
    }

    switch( opt ){
    case 'i':
      char ancfile[N_STR];
      strcpy( ancfile, argv[++opt_idx] );
      in.set_ancfile( ancfile );
      flag_optI = true;
      break;
    case 'o':
      char name[N_STR];
      strcpy( name, argv[++opt_idx] );
      in.set_name( name );
      flag_optO = true;
      break;
    case 'n':
      strcpy( tmp_str, argv[++opt_idx] );
      int num_org;
      num_org = atoi( tmp_str );
      in.set_num_org( num_org );
      flag_optN = true;
      break;
    case 's':
      strcpy( tmp_str, argv[++opt_idx] );
      int sb_size;
      sb_size = atoi( tmp_str );
      in.set_sb_size( sb_size );
      flag_optS = true;
      break;
    case 'L':
      in.set_flagL( false );
      break;
    case 'S':
      in.set_flagS( true );
      break;
    case 'D':
      in.set_flagD( true );
      break;
    case 'h':
      flag_optH = true;
      break;
    case '?':
    default:
      std::cerr << "Unknown option " << opt << std::endl;
      break;
    }
  }

  if( flag_optH || !flag_optI || !flag_optO || !flag_optN ){
    std::cerr << usage << std::endl;
    exit( 1 );
  }

  return in;
}

