#include<algorithm>
#include<iomanip>
#include<iostream>
#include<fstream>
#include<map>
#include<utility>
#include<vector>
#include<string>
#include<cstdlib>
#include<cstring>
#include<cmath>
#include<assert.h>

#define N_STR 256
#define THREE 3
#define FOUR 4
#define MAX_N 10
#define NUM_BASE 5
#define CLST_A 0.01
#define CLST_B 0.05
#define N_REPEAT 50000
#define LEARNING_REPEAT 100
#define DEFAULT_SB_SIZE 100000

template <class T> T min( T val1, T val2 ){
  return val1 < val2 ? val1 : val2;
}

template <class T> T max( T val1, T val2 ){
  return val1 > val2 ? val1 : val2;
}

class input {
  bool flagL;
  bool flagS;
  bool flagD;
  char ancfile[N_STR];
  char name[N_STR];
  char optfile[N_STR];
  char chainfile[N_STR];
  char inchainfile[N_STR];
  char chaindetailfile[N_STR];
  char osfile[N_STR];
  char inosfile[N_STR];
  char osdetailfile[N_STR];
  int num_org;
  int sb_size;
 public:
  void set_flagL( bool b ) { flagL = b; }
  void set_flagS( bool b ) { flagS = b; }
  void set_flagD( bool b ) { flagD = b; }
  void set_ancfile( char* f ) { strcpy( ancfile, f ); }
  void set_name( char* f ) {
    strcpy( name, f );
    strcpy( optfile, name ); strcat( optfile, ".options" );
    strcpy( chainfile, name ); strcat( chainfile, ".chain.txt" );
    strcpy( inchainfile, name ); strcat( inchainfile, ".anchor-in-chain.txt" );
    strcpy( chaindetailfile, name ); strcat( chaindetailfile, ".chain.detail.txt" );
    strcpy( osfile, name ); strcat( osfile, ".os.txt" );
    strcpy( inosfile, name ); strcat( inosfile, ".anchor-in-os.txt" );
    strcpy( osdetailfile, name ); strcat( osdetailfile, ".os.detail.txt" );
  }
  void set_num_org( int i ) { num_org = i; }
  void set_sb_size( int i ) { sb_size = i; }
  bool get_flagL() { return flagL; }
  bool get_flagS() { return flagS; }
  bool get_flagD() { return flagD; }
  char* get_optfile() { return optfile; }
  char* get_name() { return name; }
  char* get_ancfile() { return ancfile; }
  char* get_chainfile() { return chainfile; }
  char* get_inchainfile() { return inchainfile; }
  char* get_chaindetailfile() { return chaindetailfile; }
  char* get_osfile() { return osfile; }
  char* get_inosfile() { return inosfile; }
  char* get_osdetailfile() { return osdetailfile; }
  int get_num_org() { return num_org; }
  int get_sb_size() { return sb_size; }
  void print();
};


class block {
 public:
  int chrom_id[MAX_N];
  int start[MAX_N];
  int end[MAX_N];
  char sign[MAX_N];
  void show( int num_org );
  void show( std::ofstream fout, int num_org );
  int len;
  double score;
  double chn_scr;
  bool in_chain;
  std::vector<int> anc_idx_set;

  void cal_len( int num_org ){
    len = 0;
    for( int i=0 ; i<num_org ; ++i ){
      len += end[i] - start[i] + 1;
    }
    if( len < 0 ){
      exit(1);
    }
  }
};

class edge {
 public:
  int chrom_id[MAX_N];
  int idx1; // node index i
  int idx2; // node index j
  char sign[MAX_N];
  int len;
  double score;
  // double edge_scr;
  void show();
  void cal_len( const std::vector<block>& anc_set, int num_org ){
    len = 0;
    for( int i=0 ; i<num_org ; ++i ){
      if( sign[i] == sign[0] ){
	len += anc_set[idx2].start[i] - anc_set[idx1].end[i] + 1; 
      } else {
	len += anc_set[idx1].start[i] - anc_set[idx2].end[i] + 1;
      }
    }
    if( len < 0 ){
      std::cout << "idx1 = " << idx1 << ", idx2 = " << idx2 << std::endl;
      std::cout << len << " is not good for edge length." << std::endl; 
      exit(1);
    }
  }
};

class cluster_edge{
  std::vector<int> anc_idx_set;
 public:
  void push_anc_idx( int idx ) { anc_idx_set.push_back( idx ); }
  int get_anc_idx( int i ) { return anc_idx_set[i]; }
  std::vector<int> get_anc_idx_set() { return anc_idx_set; }
};

class point {
  double x;
  double y;
 public:
  void set_x( double d ) { x = d; }
  void set_y( double d ) { y = d; }
  double get_x() { return x; }
  double get_y() { return y; }
};

class dp_cell {
 public:
  double sin_scr;
  double chn_scr;
  int trb_idx;
  std::vector<int> anc_idx_set;
};

// functions.
// options.cpp
input option_disposal( int argc, char* argv[] );

// file.cpp
void get_anc( std::vector<block> &anc_set, char* ancfile, int num_org );

// anc_graph.cpp
void sort_anc_set( std::vector<block>& anc_set, int num_org );
int comp( block b1, block b2 );
void draw_sn_edge( std::vector<edge> &edge_set, const std::vector<block>& anc_set, int num_org, bool prog_bar=false );
bool is_same_chrom( block anc1, block anc2, int num_org );
bool is_same_chrom( block a, edge e, int num_org );
bool is_colinear( block anc1, block anc2, int num_org );
bool is_same_direc( block anc1, block anc2, int num_org );
bool is_share_space( block anc1, block anc2, int num_org );
bool is_overlap( block anc1, block anc2, int num_org );
bool is_overlap( block anc1, block anc2, int gen_idx, int num_org );
bool is_all_overlap( block anc1, block anc2, int num_org );

// learn.cpp
void initialize(double& Ps_mg, double& Ps_gm, double& Pr_mg, double& Pr_gm, const std::vector<block>& anc_set, const std::vector<edge>& sn_edge_set, int num_org);
void learn( const std::vector<edge>& sn_edge_set, const std::vector<block>& anc_set, double& Ps_mg, double& Ps_gm, double& Pr_mg, double& Pr_gm, int num_org );
void learn_anchor_model( const std::vector<block>& anc_set, double& Ps_mg, double& Pr_mg, int num_org );
void learn_edge_model( const std::vector<edge>& sn_edge_set, double& Ps_gm, double& Pr_gm, int num_org );
void sort( std::vector<int>& vec );
double cal_SHR_edge_scr( int edge_len, double s_gg_scr, double s_gm_scr, int num_org );
double cal_NSHR_edge_scr( int edge_len, double r_gg_scr, double r_gm_scr, int num_org );
double cal_SHR_anc_scr( int anc_len, double s_mm_scr, double s_mg_scr, int num_org );
double cal_NSHR_anc_scr( int anc_len, double r_mm_scr, double r_mg_scr, int num_org );
double refine_Ps_gm( std::vector<int> SHR_edge_len, int num_org );
double refine_Pr_gm( std::vector<int> NSHR_edge_len, int num_org );
double refine_Ps_mg( std::vector<int> SHR_anc_len, int num_org );
double refine_Pr_mg( std::vector<int> NSHR_anc_len, int num_org );

// score.cpp
void assignAncScr( std::vector<block>& anc_set, double ancExtend_scr, double ancOpen_scr, int numOrg );
void assignEdgeScr( std::vector<edge>& edge_set, double edgeExtend_scr, double edgeOpen_scr, int numOrg );

// chaining.cpp
void chaining_all( std::vector<block>& clst_set, const std::vector<block>& anc_set, const std::vector<edge>& sn_edge_set, int num_org );
void chaining( std::vector<block>& clst_set, const std::vector<block>& anc_set, const std::vector<edge>& sn_edge_set, int num_org );
void init_chtbl( dp_cell* dp_tbl, const std::vector<block>& anc_set );
void rfn_chtbl( dp_cell* dp_tbl, const std::vector<block>& anc_set, const std::vector<edge>& sn_edge_set, const std::vector<bool>& mask_edge, int num_org );
block trb_chtbl( const std::vector<block>& anc_set, dp_cell* dp_tbl, int num_org );
void mk_chained_anc_set( std::vector<block>& chained_anc_set, const std::vector<block>& clst_set, const std::vector<block>& anc_set );
void mk_anc_in_sb_set( std::vector<block>& anc_in_sb_set, const std::vector<block>& sb_set, const std::vector<block>& clst_set, const std::vector<block>& anc_set );
void chaining_shr( std::vector<block>& sb_set, const std::vector<block>& clst_set, int sb_size, int num_org );
void sort_edge_short2long( std::vector<edge>& shr_edge_set );
void merge_sb( std::vector<block>& sb_set, int idx1, int idx2, int num_org, std::vector<edge>& shr_edge_set, std::vector<bool>& mask_sb, std::vector<bool>& mask_edge );
bool is_large_sb( block sb, int sb_size, int num_org );
void delete_sb( std::vector<block>& sb_set, int mer_idx, int del_idx, std::vector<edge>& shr_edge_set, std::vector<bool>& mask_sb, std::vector<bool>& mask_edge );
void remove_overlap( std::vector<block>& sb_set, int mer_idx, int num_org, std::vector<edge>& shr_edge_set, std::vector<bool>& mask_sb, std::vector<bool>& mask_edge );
void remove_mask_block( std::vector<block>& sb_set, const std::vector<bool>& mask_sb );
void size_filter( std::vector<block>& sb_set, int sb_size, int num_org );
void calIdxOrder(std::multimap<double,int>& scr2idx,std::vector<block>& sb_set);

// print.cpp
void prt_anc( std::vector<block>& anc_set, int num_org );
void prt_anc( const std::vector<block>& anc_set, char* blockfile, int num_org );
void prt_chain_detail( const std::vector<block>& cls_set, const std::vector<block>& anc_set, char* blockfile, int num_org );
void prt_os_detail( const std::vector<block>& cls_set, const std::vector<block>& anc_set, char* blockfile, int num_org );
void prt_anc_len( const std::vector<block>& anc_set, char* anc_len_file );
void prt_edge_len( const std::vector<edge>& edge_set, char* edge_len_file );
void block_set_progress( int total_size, int idx, int& old_prog );
